# TODO: Package mplrs, the MPI version.

Name:           lrslib
Version:        6.2
Release:        11%{?dist}
Summary:        Reverse search for vertex enumeration/convex hull problems

%global upver 0%(sed 's/\\.//' <<< %{version})
%global lrsdir %(sed 's/[[:alpha:]]//' <<< %{upver})

License:        GPLv2+
URL:            http://cgm.cs.mcgill.ca/~avis/C/lrs.html
Source0:        http://cgm.cs.mcgill.ca/~avis/C/%{name}/archive/%{name}-%{upver}.tar.gz
# These man pages were written by Jerry James.  Text from the sources was used,
# therefore the man pages have the same copyright and license as the sources.
Source1:        %{name}-man.tar.xz
# This patch was sent upstream on 31 May 2011.  It fixes some miscellaneous
# bugs and adapts to the naming scheme we choose for installation.
Patch0:         %{name}-fixes.patch

BuildRequires:  boost-devel
BuildRequires:  gcc-c++
BuildRequires:  gmp-devel

%description
%{name} is a self-contained ANSI C implementation as a callable library
of the reverse search algorithm for vertex enumeration/convex hull
problems and comes with a choice of three arithmetic packages.  Input
file formats are compatible with Komei Fukuda's cdd package (cddlib).
All computations are done exactly in either multiple precision or fixed
integer arithmetic.  Output is not stored in memory, so even problems
with very large output sizes can sometimes be solved.

%package devel
Summary:        Header files and libraries for developing with %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}
Requires:       gmp-devel%{?_isa}

%description devel
Header files and libraries for developing with %{name}.

%package utils
Summary:        Sample programs that use %{name}
Requires:       %{name}%{?_isa} = %{version}-%{release}

%description utils
Sample programs that use %{name}.

%prep
%setup -q -n %{name}-%{lrsdir}
%setup -q -n %{name}-%{lrsdir} -T -D -a 1
%patch0

%build
# The Makefile is too primitive to use.  For one thing, it only builds
# binaries, not libraries.  We do our own thing here.

# Extract the version numbers to be used for the shared libraries.
%global ver %(sed -r 's/([\\\.[:digit:]]*)[[:alpha:]]/\\\1/' <<< %{version})
%global sover %(cut -d. -f1 <<< %{version})

CFLAGS="${RPM_OPT_FLAGS} -DTIMES -DSIGNALS -I. -I%{_includedir}/boost"
if [ %{__isa_bits} = "32" ]; then
  CFLAGS+=" -DB32"
fi

# Build the GMP version of the library
gcc $CFLAGS $RPM_LD_FLAGS -DGMP -fPIC -shared -o liblrsgmp.so.%{ver} \
    -Wl,-soname,liblrsgmp.so.%{sover} lrslib.c lrsgmp.c -lgmp
ln -s liblrsgmp.so.%{ver} liblrsgmp.so.%{sover}
ln -s liblrsgmp.so.%{sover} liblrsgmp.so

# Build the integer version of the library
gcc $CFLAGS $RPM_LD_FLAGS -DLRSLONG -fPIC -shared -o liblrslong.so.%{ver} \
    -Wl,-soname,liblrslong.so.%{sover} lrslib.c lrslong.c
ln -s liblrslong.so.%{ver} liblrslong.so.%{sover}
ln -s liblrslong.so.%{sover} liblrslong.so

# Build the multi-precision version of the library
gcc $CFLAGS $RPM_LD_FLAGS -DLRSMP -fPIC -shared -o liblrsmp.so.%{ver} \
    -Wl,-soname,liblrsmp.so.%{sover} lrslib.c lrsmp.c
ln -s liblrsmp.so.%{ver} liblrsmp.so.%{sover}
ln -s liblrsmp.so.%{sover} liblrsmp.so

# Build the nash library
gcc $CFLAGS $RPM_LD_FLAGS -DGMP -fPIC -shared -o liblrsnashlib.so.%{ver} \
    -Wl,-soname,liblrsnashlib.so.%{sover} lrsnashlib.c -L. -llrsgmp
ln -s liblrsnashlib.so.%{ver} liblrsnashlib.so.%{sover}
ln -s liblrsnashlib.so.%{sover} liblrsnashlib.so

# Build the binaries against the GMP version of the library, except rat2float
gcc $CFLAGS -DGMP lrs.c -o lrs $RPM_LD_FLAGS -L. -llrsgmp
gcc $CFLAGS -DGMP lrsnash.c -o lrsnash $RPM_LD_FLAGS -Wl,--as-needed -L. \
  -llrsnashlib -llrsgmp -lgmp
gcc $CFLAGS -DGMP fourier.c -o lrs-fourier $RPM_LD_FLAGS -L. -llrsgmp -lgmp
g++ $CFLAGS -DGMP -DPLRS plrs.cpp lrslib.c lrsgmp.c -o plrs $RPM_LD_FLAGS \
    -L. -llrsgmp -lgmp -lboost_thread
gcc $CFLAGS -DGMP redund.c -o lrs-redund $RPM_LD_FLAGS -L. -llrsgmp
gcc $CFLAGS -DGMP setupnash.c -o lrs-setnash $RPM_LD_FLAGS -L. -llrsgmp
gcc $CFLAGS -DGMP setupnash2.c -o lrs-setnash2 $RPM_LD_FLAGS -L. -llrsgmp
gcc $CFLAGS -DLRSMP rat2float.c -o lrs-rat2float $RPM_LD_FLAGS -L. -llrsmp
gcc $CFLAGS float2rat.c -o lrs-float2rat $RPM_LD_FLAGS
gcc $CFLAGS 2nash.c -o lrs-2nash $RPM_LD_FLAGS

%install
# Install the libraries
mkdir -p $RPM_BUILD_ROOT%{_libdir}
cp -a liblrs*.so* $RPM_BUILD_ROOT%{_libdir}
chmod 0755 $RPM_BUILD_ROOT%{_libdir}/lib*.so.%{ver}

# Install the binaries
mkdir -p $RPM_BUILD_ROOT%{_bindir}
install -p -m 0755 lrs plrs lrs-* $RPM_BUILD_ROOT%{_bindir}

# Install the header files, but alter them to reflect 32-/64-bitness and fix
# up the include directives.
mkdir -p $RPM_BUILD_ROOT%{_includedir}/%{name}
if [ %{__isa_bits} = "64" ]; then
  sed -r -e 's|"(lrs.*\.h)"|<lrslib/\1>|' \
      -e "s|#include ARITH|#define B64\\n#include ARITH|" \
      lrslib.h > $RPM_BUILD_ROOT%{_includedir}/%{name}/lrslib.h
else
  sed -r 's|"(lrs.*\.h)"|<lrslib/\1>|' lrslib.h > \
      $RPM_BUILD_ROOT%{_includedir}/%{name}/lrslib.h
fi
touch -r lrslib.h $RPM_BUILD_ROOT%{_includedir}/%{name}/lrslib.h

sed -e 's|"gmp.h"|<gmp.h>|' lrsgmp.h > \
    $RPM_BUILD_ROOT%{_includedir}/%{name}/lrsgmp.h
touch -r lrsgmp.h $RPM_BUILD_ROOT%{_includedir}/%{name}/lrsgmp.h

cp -p lrslong.h lrsnashlib.h lrsmp.h $RPM_BUILD_ROOT%{_includedir}/%{name}

# Install the man pages
mkdir -p $RPM_BUILD_ROOT%{_mandir}/man1
cd man
for f in *.1; do
  sed "s/@VERSION@/%{upver}/" $f > $RPM_BUILD_ROOT%{_mandir}/man1/$f
  touch -r $f $RPM_BUILD_ROOT%{_mandir}/man1/$f
done

%ldconfig_scriptlets

%files
%doc README
%license COPYING
%{_libdir}/*.so.*

%files devel
%doc chdemo.c lpdemo.c vedemo.c
%{_includedir}/%{name}
%{_libdir}/*.so

%files utils
%{_bindir}/lrs*
%{_bindir}/plrs
%{_mandir}/man1/lrs*
%{_mandir}/man1/plrs*

%changelog
* Fri Jul 13 2018 Fedora Release Engineering <releng@fedoraproject.org> - 6.2-11
- Rebuilt for https://fedoraproject.org/wiki/Fedora_29_Mass_Rebuild

* Sat Mar  3 2018 Jerry James <loganjerry@gmail.com> - 6.2-10
- BR gcc-c++ instead of gcc

* Thu Feb 08 2018 Fedora Release Engineering <releng@fedoraproject.org> - 6.2-9
- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild

* Thu Feb  1 2018 Jerry James <loganjerry@gmail.com> - 6.2-8
- Rebuild for Boost 1.66

* Thu Aug 03 2017 Fedora Release Engineering <releng@fedoraproject.org> - 6.2-7
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Binutils_Mass_Rebuild

* Wed Jul 26 2017 Fedora Release Engineering <releng@fedoraproject.org> - 6.2-6
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Mass_Rebuild

* Wed Jul 19 2017 Jonathan Wakely <jwakely@redhat.com> - 6.2-5
- Rebuilt for s390x binutils bug

* Mon Jul 03 2017 Jonathan Wakely <jwakely@redhat.com> - 6.2-4
- Rebuilt for Boost 1.64

* Fri Feb 10 2017 Fedora Release Engineering <releng@fedoraproject.org> - 6.2-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_Mass_Rebuild

* Fri Jan 27 2017 Jonathan Wakely <jwakely@redhat.com> - 6.2-2
- Rebuilt for Boost 1.63

* Fri May 27 2016 Jerry James <loganjerry@gmail.com> - 6.2-1
- New upstream release

* Tue May 17 2016 Jonathan Wakely <jwakely@redhat.com> - 6.1-4
- Rebuilt for linker errors in boost (#1331983)

* Thu Feb 04 2016 Fedora Release Engineering <releng@fedoraproject.org> - 6.1-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_24_Mass_Rebuild

* Fri Jan 15 2016 Jonathan Wakely <jwakely@redhat.com> - 6.1-2
- Rebuilt for Boost 1.60

* Fri Dec  4 2015 Jerry James <jamesjer@diannao.jamezone.org> - 6.1-1
- New upstream release

* Sat Sep 19 2015 Jerry James <loganjerry@gmail.com> - 6.0-1
- New upstream release

* Thu Aug 27 2015 Jonathan Wakely <jwakely@redhat.com> - 5.1-7
- Rebuilt for Boost 1.59

* Wed Jul 29 2015 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 5.1-6
- Rebuilt for https://fedoraproject.org/wiki/Changes/F23Boost159

* Wed Jul 22 2015 David Tardon <dtardon@redhat.com> - 5.1-5
- rebuild for Boost 1.58

* Wed Jun 17 2015 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 5.1-4
- Rebuilt for https://fedoraproject.org/wiki/Fedora_23_Mass_Rebuild

* Sat May 02 2015 Kalev Lember <kalevlember@gmail.com> - 5.1-3
- Rebuilt for GCC 5 C++11 ABI change

* Wed Feb 04 2015 Petr Machata <pmachata@redhat.com> - 5.1-2
- Bump for rebuild.

* Mon Feb  2 2015 Jerry James <loganjerry@gmail.com> - 5.1-1
- New upstream release

* Mon Jan 26 2015 Petr Machata <pmachata@redhat.com> - 5.0a-2
- Rebuild for boost 1.57.0

* Thu Jan 15 2015 Jerry James <loganjerry@gmail.com> - 5.0a-1
- New upstream release

* Mon Nov 10 2014 Jerry James <loganjerry@gmail.com> - 5.0-1
- New upstream release
- Drop upstreamed -memleak patch
- Link with RPM_LD_FLAGS
- Fix license handling

* Sun Aug 17 2014 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.3-5
- Rebuilt for https://fedoraproject.org/wiki/Fedora_21_22_Mass_Rebuild

* Sat Jun 07 2014 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.3-4
- Rebuilt for https://fedoraproject.org/wiki/Fedora_21_Mass_Rebuild

* Sat Aug 03 2013 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.3-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_20_Mass_Rebuild

* Thu Feb 14 2013 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.3-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_19_Mass_Rebuild

* Mon Aug 13 2012 Jerry James <loganjerry@gmail.com> - 4.3-1
- New upstream release

* Thu Jul 19 2012 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.2c-5
- Rebuilt for https://fedoraproject.org/wiki/Fedora_18_Mass_Rebuild

* Fri Mar 16 2012 Jerry James <loganjerry@gmail.com> - 4.2c-4
- Apply patch from Thomas Rehn to fix a memory leak

* Tue Feb 14 2012 Jerry James <loganjerry@gmail.com> - 4.2c-3
- Change subpackage structure based on review

* Wed Aug 24 2011 Jerry James <loganjerry@gmail.com> - 4.2c-2
- Use %%{name} more liberally.
- Use %%global instead of %%define.
- Add man pages.

* Wed Jul 20 2011 Jerry James <loganjerry@gmail.com> - 4.2c-1
- Initial RPM
